#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Tests for Wiki models

"""

import os
import shutil
import unittest
import pandas as pd
from ethnicolr.pred_wiki_ln import pred_wiki_ln
from ethnicolr.pred_wiki_name import pred_wiki_name
from . import capture

race = [
    "Asian,GreaterEastAsian,EastAsian",
    "Asian,GreaterEastAsian,Japanese",
    "Asian,IndianSubContinent",
    "GreaterAfrican,Africans",
    "GreaterAfrican,Muslim",
    "GreaterEuropean,British",
    "GreaterEuropean,EastEuropean",
    "GreaterEuropean,Jewish",
    "GreaterEuropean,WestEuropean,French",
    "GreaterEuropean,WestEuropean,Germanic",
    "GreaterEuropean,WestEuropean,Hispanic",
    "GreaterEuropean,WestEuropean,Italian",
    "GreaterEuropean,WestEuropean,Nordic",
]

race_mean = [
    "Asian,GreaterEastAsian,EastAsian_mean",
    "Asian,GreaterEastAsian,Japanese_mean",
    "Asian,IndianSubContinent_mean",
    "GreaterAfrican,Africans_mean",
    "GreaterAfrican,Muslim_mean",
    "GreaterEuropean,British_mean",
    "GreaterEuropean,EastEuropean_mean",
    "GreaterEuropean,Jewish_mean",
    "GreaterEuropean,WestEuropean,French_mean",
    "GreaterEuropean,WestEuropean,Germanic_mean",
    "GreaterEuropean,WestEuropean,Hispanic_mean",
    "GreaterEuropean,WestEuropean,Italian_mean",
    "GreaterEuropean,WestEuropean,Nordic_mean",
]


class TestPredWiki(unittest.TestCase):
    def setUp(self):
        names = [
            {"last": "smith", "first": "john",
             "true_race": "GreaterEuropean,British"},
            {
                "last": "zhang",
                "first": "simon",
                "true_race": "Asian,GreaterEastAsian,EastAsian",
            },
        ]
        self.df = pd.DataFrame(names)

    def tearDown(self):
        pass

    def test_pred_wiki_ln(self):
        odf = pred_wiki_ln(self.df, "last")
        self.assertTrue(
            all(
                odf[[col for col in odf.columns
                     if col in race]].sum(axis=1).round(1)
                == 1.0
            )
        )
        self.assertTrue(all(odf.true_race == odf.race))

    def test_pred_wiki_name(self):
        odf = pred_wiki_name(self.df, "last", "first")
        self.assertTrue(
            all(
                odf[[col for col in odf.columns
                     if col in race]].sum(axis=1).round(1)
                == 1.0
            )
        )
        self.assertTrue(all(odf.true_race == odf.race))

    def test_pred_wiki_ln_mean(self):
        odf = pred_wiki_ln(self.df, "last", conf_int=0.9)
        self.assertTrue(
            all(
                odf[[col for col in odf.columns
                     if col in race_mean]].sum(axis=1).round(1)
                == 1.0
            )
        )
        self.assertTrue(all(odf.true_race == odf.race))

    def test_pred_wiki_name_mean(self):
        odf = pred_wiki_name(self.df, "last", "first", conf_int=0.9)
        self.assertTrue(
            all(
                odf[[col for col in odf.columns
                     if col in race_mean]].sum(axis=1).round(1)
                == 1.0
            )
        )
        self.assertTrue(all(odf.true_race == odf.race))

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import argparse
import pandas as pd
import numpy as np
from tensorflow.keras.models import load_model
from tensorflow.keras.preprocessing import sequence

from pkg_resources import resource_filename

from .utils import column_exists, fixup_columns, transform_and_pred

MODELFN = "models/nc_voter_reg/lstm/nc_voter_name_lstm_oversample.h5"
VOCABFN = "models/nc_voter_reg/lstm/nc_voter_name_vocab_oversample.csv"
RACEFN = "models/nc_voter_reg/lstm/nc_name_race.csv"

MODEL = resource_filename(__name__, MODELFN)
VOCAB = resource_filename(__name__, VOCABFN)
RACE = resource_filename(__name__, RACEFN)

NGRAMS = (2, 3)
FEATURE_LEN = 25


class NCRegNameModel():
    vocab = None
    race = None
    model = None

    @classmethod
    def pred_nc_reg_name(cls, df, lname_col, fname_col, num_iter=100,
                         conf_int=1.0):
        """Predict the race+ethnicity by the full name using
        North Carolina 12 category voter model.

        Using the NC voter full name model to predict the race/ethnicity of
        the input DataFrame.

        Args:
            df (:obj:`DataFrame`): Pandas DataFrame containing the last name
                and first name column.
            lname_col (str or int): Column's name or location of the last name
                in DataFrame.
            fname_col (str or int): Column's name or location of the first name
                in DataFrame.

        Returns:
            DataFrame: Pandas DataFrame with additional columns:
                - `race` the predict result
                - Additional columns for probability of each classes.

        """

        if lname_col not in df.columns:
            print("No column `{0!s}` in the DataFrame".format(lname_col))
            return df
        if fname_col not in df.columns:
            print("No column `{0!s}` in the DataFrame".format(fname_col))
            return df

        df['__name'] = (df[lname_col].str.strip()
                        + ' ' + df[fname_col].str.strip()).str.title()

        df.dropna(subset=['__name'])
        if df.shape[0] == 0:
            del df['__name']
            return df

        rdf = transform_and_pred(df=df,
                                 newnamecol='__name',
                                 cls=cls,
                                 VOCAB=VOCAB,
                                 RACE=RACE,
                                 MODEL=MODEL,
                                 NGRAMS=NGRAMS,
                                 maxlen=FEATURE_LEN,
                                 num_iter=num_iter,
                                 conf_int=conf_int)

        return rdf


pred_nc_reg_name = NCRegNameModel.pred_nc_reg_name


def main(argv=sys.argv[1:]):
    title = ('Predict Race/Ethnicity by name using NC 12 category'
             ' voter registration model')
    parser = argparse.ArgumentParser(description=title)
    parser.add_argument('input', default=None,
                        help='Input file')
    parser.add_argument('-o', '--output', default='nc-pred-name-output.csv',
                        help='Output file with prediction data')
    parser.add_argument('-f', '--first', required=True,
                        help='Name or index location of column contains '
                             'the first name')
    parser.add_argument('-l', '--last', required=True,
                        help='Name or index location of column contains '
                             'the last name')
    parser.add_argument('-i', '--iter', default=100, type=int,
                        help='Number of iterations to measure uncertainty')
    parser.add_argument('-c', '--conf', default=1.0, type=float,
                        help='Confidence interval of Predictions')

    args = parser.parse_args(argv)

    print(args)

    if not args.last.isdigit() and not args.first.isdigit():
        df = pd.read_csv(args.input)
    else:
        df = pd.read_csv(args.input, header=None)
        args.last = int(args.last)
        args.first = int(args.first)

    if not column_exists(df, args.last):
        return -1
    if not column_exists(df, args.first):
        return -1

    rdf = pred_nc_reg_name(df, args.last, args.first, args.iter, args.conf)

    print("Saving output to file: `{0:s}`".format(args.output))
    rdf.columns = fixup_columns(rdf.columns)
    rdf.to_csv(args.output, index=False)

    return 0


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
import sys

import numpy as np
import pandas as pd
from tensorflow.keras.models import load_model
from tensorflow.keras.preprocessing import sequence
from pkg_resources import resource_filename

from ethnicolr.utils import (column_exists, find_ngrams, fixup_columns,
                             transform_and_pred)

MODELFN = "models/fl_voter_reg/lstm/fl_all_ln_lstm_uncrtn.h5"
VOCABFN = "models/fl_voter_reg/lstm/fl_all_ln_vocab.csv"
RACEFN = "models/fl_voter_reg/lstm/fl_ln_race.csv"

MODEL = resource_filename(__name__, MODELFN)
VOCAB = resource_filename(__name__, VOCABFN)
RACE = resource_filename(__name__, RACEFN)

NGRAMS = 2
FEATURE_LEN = 20


class FloridaRegLnCnIntModel():
    vocab = None
    race = None
    model = None

    @classmethod
    def pred_fl_reg_ln_cn_int(cls, df, namecol, num_iter=100, conf_int=1.0):
        """Predict the race/ethnicity by the last name using Florida voter
        model.

        Using the Florida voter last name model to predict the race/ethnicity
        of the input DataFrame.

        Args:
            df (:obj:`DataFrame`): Pandas DataFrame containing the last name
                column.
            namecol (str or int): Column's name or location of the name in
                DataFrame.
            num_iter (int): Number of iterations/Predictions for each entry
                that determines model uncertainity
            conf_int (float): confidence interval of the prediction made

        Returns:
            DataFrame: Pandas DataFrame with additional columns:
                - `race` the predict result
                - probability of the class predicted by the model
                - standard error of the prediction made by the model
                - confidence interval as list [lower quantile, upper quantile]
                - Additional columns for probability of each classes.

        """

        if namecol not in df.columns:
            print("No column `{0!s}` in the DataFrame".format(namecol))
            return df

        nn = df[namecol].notnull()
        if df[nn].shape[0] == 0:
            return df

        df['__last_name'] = df[namecol].str.strip()
        df['__last_name'] = df['__last_name'].str.title()

        if cls.model is None:
            #  sort n-gram by freq (highest -> lowest)
            vdf = pd.read_csv(VOCAB)
            cls.vocab = vdf.vocab.tolist()

            rdf = pd.read_csv(RACE)
            cls.race = rdf.race.tolist()

            cls.model = load_model(MODEL)

        # build X from index of n-gram sequence
        X = np.array(df[nn]['__last_name'].apply(lambda c:
                                                 find_ngrams(cls.vocab,
                                                             c, NGRAMS)))
        X = sequence.pad_sequences(X, maxlen=FEATURE_LEN)

        # define the quantile ranges for the confidence interval
        low_quantile = 0.5 - (conf_int / 2)
        high_quantile = 0.5 + (conf_int / 2)

        # get num_iter predictions to be able to measure model uncertainty
        proba = []
        for _ in range(num_iter):
            proba.append(cls.model.predict(X, verbose=2))

        # creating arrays for the confidence interval analysis:
        #   1 - mean of the predictions
        #   2 - std deviation of the predictions
        #   3 - lower quantile of the predictions
        #   4 - upper quantile of the predictions
        proba = np.array(proba)
        mean_arr = proba.mean(axis=0).reshape(-1, len(cls.race))
        std_arr = proba.std(axis=0).reshape(-1, len(cls.race))
        pct_low_arr = np.quantile(proba, low_quantile,
                                  axis=0).reshape(-1, len(cls.race))
        pct_high_arr = np.quantile(proba, high_quantile,
                                   axis=0).reshape(-1, len(cls.race))

        df.loc[nn, '__pred'] = np.argmax(mean_arr, axis=-1)

        df.loc[nn, 'race'] = df[nn]['__pred'].apply(lambda c:
                                                    cls.race[int(c)])
        stats = np.zeros((df.shape[0], 4))
        conf_int = []

        # selecting the statistics of the chosen class
        for i in range(df.shape[0]):
            select_class = np.argmax(mean_arr[i], axis=-1)
            stats[i, 0] = mean_arr[i, select_class]
            stats[i, 1] = std_arr[i, select_class]
            stats[i, 2] = pct_low_arr[i, select_class]
            stats[i, 3] = pct_high_arr[i, select_class]
            conf_int.append(np.array([stats[i, 2], stats[i, 3]]).tolist())

        df['proba'] = stats[:, 0]
        df['std_err'] = stats[:, 1]
        df['conf_int'] = conf_int

        # take out temporary working columns
        del df['__pred']
        del df['__last_name']

        pdf = pd.DataFrame(mean_arr, columns=cls.race)
        pdf.set_index(df[nn].index, inplace=True)

        rdf = pd.concat([df, pdf], axis=1)

        return rdf


pred_fl_reg_ln_cn_int = FloridaRegLnCnIntModel.pred_fl_reg_ln_cn_int


def main(argv=sys.argv[1:]):
    title = ('Predict Race/Ethnicity by name using Florida registration model'
             ' with Confidence Interval')
    parser = argparse.ArgumentParser(description=title)
    parser.add_argument('input', default=None,
                        help='Input file')
    parser.add_argument('-o', '--output', default='fl-pred-ln-conf-output.csv',
                        help='Output file with prediction data')
    parser.add_argument('-i', '--iter', default=100, type=int,
                        help='Number of iterations to measure uncertainty')
    parser.add_argument('-c', '--conf', default=1.0, type=float,
                        help='Confidence interval of Predictions')
    parser.add_argument('-l', '--last', required=True,
                        help='Name or index location of column contains '
                             'the last name')

    args = parser.parse_args(argv)

    print(args)

    if not args.last.isdigit():
        df = pd.read_csv(args.input)
    else:
        df = pd.read_csv(args.input, header=None)
        args.last = int(args.last)

    if not column_exists(df, args.last):
        return -1

    rdf = pred_fl_reg_ln_cn_int(df, args.last, args.iter, args.conf)

    print("Saving output to file: `{0:s}`".format(args.output))
    rdf.columns = fixup_columns(rdf.columns)
    rdf.to_csv(args.output, index=False)

    return 0


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Tests for FL voter registration models

"""

import os
import shutil
import unittest
import pandas as pd
from ethnicolr.pred_fl_reg_ln import pred_fl_reg_ln
from ethnicolr.pred_fl_reg_name import pred_fl_reg_name
from ethnicolr.pred_fl_reg_ln_five_cat import pred_fl_reg_ln_five_cat
from ethnicolr.pred_fl_reg_name_five_cat import pred_fl_reg_name_five_cat
from pkg_resources import resource_filename

from . import capture

race = ["asian", "hispanic", "nh_black", "nh_white"]
race5 = ["asian", "hispanic", "nh_black", "nh_white", "other"]

race_mean = ["asian_mean", "hispanic_mean", "nh_black_mean", "nh_white_mean"]
race5_mean = ["asian_mean", "hispanic_mean", "nh_black_mean", "nh_white_mean",
              "other_mean"]


class TestPredFL(unittest.TestCase):
    def setUp(self):
        names = [
            {"last": "zhang", "first": "simon", "true_race": "asian"},
            {"last": "torres", "first": "raul", "true_race": "hispanic"},
        ]
        self.df = pd.DataFrame(names)

    def tearDown(self):
        pass

    def test_pred_fl_reg_ln(self):
        odf = pred_fl_reg_ln(self.df, "last")
        self.assertTrue(
            all(
                odf[[col for col in odf.columns
                     if col in race]].sum(axis=1).round(1)
                == 1.0
            )
        )
        self.assertTrue(all(odf.true_race == odf.race))

    def test_pred_fl_reg_name(self):
        odf = pred_fl_reg_name(self.df, "last", "first")
        self.assertTrue(
            all(
                odf[[col for col in odf.columns
                     if col in race]].sum(axis=1).round(1)
                == 1.0
            )
        )
        self.assertTrue(all(odf.true_race == odf.race))

    def test_pred_fl_reg_ln_five_cat(self):
        odf = pred_fl_reg_ln_five_cat(self.df, "last")
        self.assertTrue(
            all(
                odf[[col for col in odf.columns
                     if col in race5]].sum(axis=1).round(1)
                == 1.0
            )
        )
        self.assertTrue(all(odf.true_race == odf.race))

    def test_pred_fl_reg_name_five_cat(self):
        odf = pred_fl_reg_name_five_cat(self.df, "last", "first")
        self.assertTrue(
            all(
                odf[[col for col in odf.columns
                     if col in race5]].sum(axis=1).round(1)
                == 1.0
            )
        )
        self.assertTrue(all(odf.true_race == odf.race))

    def test_pred_fl_reg_ln_mean(self):
        odf = pred_fl_reg_ln(self.df, "last", conf_int=0.9)
        self.assertTrue(
            all(
                odf[[col for col in odf.columns
                     if col in race_mean]].sum(axis=1).round(1)
                == 1.0
            )
        )
        self.assertTrue(all(odf.true_race == odf.race))

    def test_pred_fl_reg_name_mean(self):
        odf = pred_fl_reg_name(self.df, "last", "first", conf_int=0.9)
        self.assertTrue(
            all(
                odf[[col for col in odf.columns
                     if col in race_mean]].sum(axis=1).round(1)
                == 1.0
            )
        )
        self.assertTrue(all(odf.true_race == odf.race))

    def test_pred_fl_reg_ln_five_cat_mean(self):
        odf = pred_fl_reg_ln_five_cat(self.df, "last", conf_int=0.9)
        self.assertTrue(
            all(
                odf[[col for col in odf.columns
                     if col in race5_mean]].sum(axis=1).round(1)
                == 1.0
            )
        )
        self.assertTrue(all(odf.true_race == odf.race))

    def test_pred_fl_reg_name_five_cat_mean(self):
        odf = pred_fl_reg_name_five_cat(self.df, "last", "first", conf_int=0.9)
        self.assertTrue(
            all(
                odf[[col for col in odf.columns
                     if col in race5_mean]].sum(axis=1).round(1)
                == 1.0
            )
        )
        self.assertTrue(all(odf.true_race == odf.race))

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Tests for census_ln.py

"""

import os
import shutil
import unittest
import pandas as pd
from ethnicolr.census_ln import census_ln
from . import capture


class TestCensusLn(unittest.TestCase):

    def setUp(self):
        names = [{'last': 'smith', 'true_race': 'white'},
                 {'last': 'zhang', 'true_race': 'api'}]
        self.df = pd.DataFrame(names)

    def tearDown(self):
        pass

    def test_census_ln_2000(self):
        odf = census_ln(self.df, 'last', 2000)
        self.assertIn('pctwhite', odf.columns)

    def test_census_ln_2010(self):
        odf = census_ln(self.df, 'last', 2010)
        self.assertIn('pcthispanic', odf.columns)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

from re import VERBOSE
import sys
import pandas as pd
import numpy as np
from tensorflow.keras.models import load_model
from tensorflow.keras.preprocessing import sequence
from pkg_resources import resource_filename
from itertools import chain


def isstring(s):
    # if we use Python 3
    if sys.version_info[0] >= 3:
        return isinstance(s, str)
    # we use Python 2
    return isinstance(s, basestring)


def column_exists(df, col):
    """Check the column name exists in the DataFrame.

    Args:
        df (:obj:`DataFrame`): Pandas DataFrame.
        col (str): Column name.

    Returns:
        bool: True if exists, False if not exists.

    """
    if col and (col not in df.columns):
        print("The specify column `{0!s}` not found in the input file"
              .format(col))
        return False
    else:
        return True


def fixup_columns(cols):
    """Replace index location column to name with `col` prefix

    Args:
        cols (list): List of original columns

    Returns:
        list: List of column names

    """
    out_cols = []
    for col in cols:
        if type(col) == int:
            out_cols.append("col{:d}".format(col))
        else:
            out_cols.append(col)
    return out_cols


def n_grams(seq, n=1):
    """Returns an itirator over the n-grams given a listTokens"""
    shiftToken = lambda i: (el for j,el in enumerate(seq) if j>=i)
    shiftedTokens = (shiftToken(i) for i in range(n))
    tupleNGrams = zip(*shiftedTokens)
    return tupleNGrams


def range_ngrams(listTokens, ngramRange=(1,2)):
    """Returns an itirator over all n-grams for n in range(ngramRange)
       given a listTokens.
    """

    ngrams = (ngramRange[0], ngramRange[1] + 1)
    return chain(*(n_grams(listTokens, i) for i in range(*ngramRange)))


def find_ngrams(vocab, text, n):
    """Find and return list of the index of n-grams in the vocabulary list.

    Generate the n-grams of the specific text, find them in the vocabulary list
    and return the list of index have been found.

    Args:
        vocab (:obj:`list`): Vocabulary list.
        text (str): Input text
        n (int or tuple): N-grams or tuple of range N-grams

    Returns:
        list: List of the index of n-grams in the vocabulary list.

    """

    wi = []

    if not isstring(text):
        return wi

    if type(n) is tuple:
        a = range_ngrams(text, n)
    else:
        a = zip(*[text[i:] for i in range(n)])

    for i in a:
        w = "".join(i)
        try:
            idx = vocab.index(w)
        except Exception as e:
            idx = 0
        wi.append(idx)
    return wi


def transform_and_pred(
    df, newnamecol, cls, VOCAB, RACE, MODEL, NGRAMS, maxlen, num_iter, conf_int
):

    df[newnamecol] = df[newnamecol].str.strip().str.title()
    df["rowindex"] = df.index

    if cls.model is None:
        vdf = pd.read_csv(VOCAB)
        cls.vocab = vdf.vocab.tolist()

        rdf = pd.read_csv(RACE)
        cls.race = rdf.race.tolist()

        cls.model = load_model(MODEL)

    # build X from index of n-gram sequence
    X = np.array(df[newnamecol].apply(lambda c: find_ngrams(cls.vocab,
                                                            c, NGRAMS)))
    X = sequence.pad_sequences(X, maxlen=maxlen)

    if conf_int == 1:
        # Predict
        proba = cls.model(X, training=False).numpy()
        pdf = pd.DataFrame(proba, columns=cls.race)
        pdf["__race"] = np.argmax(proba, axis=-1)
        pdf["race"] = pdf["__race"].apply(lambda c: cls.race[int(c)])
        del pdf["__race"]
        final_df = pd.concat([df.reset_index(drop=True),
                              pdf.reset_index(drop=True)], axis=1 )
    else:
        # define the quantile ranges for the confidence interval
        lower_perc = 0.5 - (conf_int / 2)
        upper_perc = 0.5 + (conf_int / 2)

        # Predict
        pdf = pd.DataFrame()

        for _ in range(num_iter):
            pdf = pd.concat([pdf, pd.DataFrame(cls.model(X, training=True))])
        print(cls.race)
        pdf.columns = cls.race
        pdf["rowindex"] = pdf.index

        res = (
            pdf.groupby("rowindex")
            .agg(
                [
                    np.mean,
                    np.std,
                    lambda x: np.percentile(x, q=lower_perc),
                    lambda x: np.percentile(x, q=upper_perc),
                ]
            )
            .reset_index()
        )
        res.columns = [f"{i}_{j}" for i, j in res.columns]
        res.columns = res.columns.str.replace("<lambda_0>", "lb")
        res.columns = res.columns.str.replace("<lambda_1>", "ub")
        res.columns = res.columns.str.replace("rowindex_", "rowindex")

        means = list(filter(lambda x: "_mean" in x, res.columns))
        res["race"] = res[means].idxmax(axis=1).str.replace("_mean", "")

        for suffix in ["_lb", "ub"]:
            conv_filt = list(filter(lambda x: suffix in x, res.columns))
            res[conv_filt] = res[conv_filt].to_numpy().astype(float)

        final_df = df.merge(res, on="rowindex", how="left")

    del final_df['rowindex']
    del df['rowindex']

    return final_df



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import argparse
import pandas as pd
import numpy as np
from tensorflow.keras.models import load_model
from tensorflow.keras.preprocessing import sequence
from pkg_resources import resource_filename

from .utils import column_exists, fixup_columns, transform_and_pred

MODELFN = "models/census/lstm/census{0:d}_ln_lstm.h5"
VOCABFN = "models/census/lstm/census{0:d}_ln_vocab.csv"
RACEFN = "models/census/lstm/census{0:d}_race.csv"

MODEL = resource_filename(__name__, MODELFN)
VOCAB = resource_filename(__name__, VOCABFN)
RACE = resource_filename(__name__, RACEFN)

NGRAMS = 2
FEATURE_LEN = 20


class CensusLnModel:
    vocab = None
    race = None
    model = None
    model_year = None

    @classmethod
    def pred_census_ln(cls, df, namecol, year=2000, num_iter=100,
                       conf_int=1.0):
        """Predict the race/ethnicity by the last name using Census model.

        Using the Census last name model to predict the race/ethnicity of the
        input DataFrame.

        Args:
            df (:obj:`DataFrame`): Pandas DataFrame containing the last name
                column.
            namecol (str or int): Column's name or location of the name in
                DataFrame.
            year (int): The year of Census model to be used. (2000 or 2010)
                (default is 2000)

        Returns:
            DataFrame: Pandas DataFrame with additional columns:
                - `race` the predict result
                - `black`, `api`, `white`, `hispanic` are the prediction
                    probability.

        """

        if namecol not in df.columns:
            print("No column `{0!s}` in the DataFrame".format(namecol))
            return df

        df.dropna(subset=[namecol])
        if df.shape[0] == 0:
            return df

        VOCAB = resource_filename(__name__, VOCABFN.format(year))
        MODEL = resource_filename(__name__, MODELFN.format(year))
        RACE = resource_filename(__name__, RACEFN.format(year))

        rdf = transform_and_pred(
            df=df,
            newnamecol=namecol,
            cls=cls,
            VOCAB=VOCAB,
            RACE=RACE,
            MODEL=MODEL,
            NGRAMS=NGRAMS,
            maxlen=FEATURE_LEN,
            num_iter=num_iter,
            conf_int=conf_int,
        )

        return rdf


pred_census_ln = CensusLnModel.pred_census_ln


def main(argv=sys.argv[1:]):
    title = "Predict Race/Ethnicity by last name using Census model"
    parser = argparse.ArgumentParser(description=title)
    parser.add_argument("input", default=None, help="Input file")
    parser.add_argument(
        "-y",
        "--year",
        type=int,
        default=2000,
        choices=[2000, 2010],
        help="Year of Census data (default=2000)",
    )
    parser.add_argument(
        "-o",
        "--output",
        default="census-pred-ln-output.csv",
        help="Output file with prediction data",
    )
    parser.add_argument(
        "-l",
        "--last",
        required=True,
        help="Name or index location of column contains " "the last name",
    )
    parser.add_argument('-i', '--iter', default=100, type=int,
                        help='Number of iterations to measure uncertainty')
    parser.add_argument('-c', '--conf', default=1.0, type=float,
                        help='Confidence interval of Predictions')

    args = parser.parse_args(argv)

    print(args)

    if not args.last.isdigit():
        df = pd.read_csv(args.input)
    else:
        df = pd.read_csv(args.input, header=None)
        args.last = int(args.last)

    if not column_exists(df, args.last):
        return -1

    rdf = pred_census_ln(df, args.last, args.year, args.iter, args.conf)

    print("Saving output to file: `{0:s}`".format(args.output))
    rdf.columns = fixup_columns(rdf.columns)
    rdf.to_csv(args.output, index=False)

    return 0


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Tests for NC voter registration models

"""

import os
import shutil
import unittest
import pandas as pd
from ethnicolr.pred_nc_reg_name import pred_nc_reg_name

from . import capture

race = [
    "HL+A",
    "HL+B",
    "HL+I",
    "HL+M",
    "HL+O",
    "HL+W",
    "NL+A",
    "NL+B",
    "NL+I",
    "NL+M",
    "NL+O",
    "NL+W",
]

race_mean = [
    "HL+A_mean",
    "HL+B_mean",
    "HL+I_mean",
    "HL+M_mean",
    "HL+O_mean",
    "HL+W_mean",
    "NL+A_mean",
    "NL+B_mean",
    "NL+I_mean",
    "NL+M_mean",
    "NL+O_mean",
    "NL+W_mean",
]


class TestPredNC(unittest.TestCase):
    def setUp(self):
        names = [
            {"last": "hernandez", "first": "hector", "true_race": "HL+O"},
            {"last": "zhang", "first": "simon", "true_race": "NL+A"},
        ]
        self.df = pd.DataFrame(names)

    def tearDown(self):
        pass

    def test_pred_nc_reg_name(self):
        odf = pred_nc_reg_name(self.df, "last", "first")
        self.assertTrue(
            all(
                odf[[col for col in odf.columns
                     if col in race]].sum(axis=1).round(1)
                == 1.0
            )
        )
        self.assertTrue(all(odf.true_race == odf.race))

    def test_pred_nc_reg_name(self):
        odf = pred_nc_reg_name(self.df, "last", "first", conf_int=0.9)
        self.assertTrue(
            all(
                odf[[col for col in odf.columns
                     if col in race_mean]].sum(axis=1).round(1)
                == 1.0
            )
        )
        self.assertTrue(all(odf.true_race == odf.race))

if __name__ == "__main__":
    unittest.main()




from ethnicolr.census_ln import census_ln
from ethnicolr.pred_census_ln import pred_census_ln
from ethnicolr.pred_wiki_ln import pred_wiki_ln
from ethnicolr.pred_wiki_name import pred_wiki_name
from ethnicolr.pred_fl_reg_ln import pred_fl_reg_ln
from ethnicolr.pred_fl_reg_ln_cn_int import pred_fl_reg_ln_cn_int
from ethnicolr.pred_fl_reg_name import pred_fl_reg_name
from ethnicolr.pred_nc_reg_name import pred_nc_reg_name
from ethnicolr.pred_fl_reg_ln_five_cat import pred_fl_reg_ln_five_cat
from ethnicolr.pred_fl_reg_name_five_cat import pred_fl_reg_name_five_cat


__all__ = ['census_ln', 'pred_census_ln', 'pred_wiki_ln', 'pred_wiki_name',
           'pred_fl_reg_ln', 'pred_fl_reg_ln_cn_int', 'pred_fl_reg_name',
           'pred_nc_reg_name', 'pred_fl_reg_ln_five_cat',
           'pred_fl_reg_name_five_cat']



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
import sys

import numpy as np
import pandas as pd
from tensorflow.keras.models import load_model
from tensorflow.keras.preprocessing import sequence
from pkg_resources import resource_filename

from .utils import column_exists, fixup_columns, transform_and_pred

MODELFN = "models/fl_voter_reg/lstm/fl_all_ln_lstm.h5"
VOCABFN = "models/fl_voter_reg/lstm/fl_all_ln_vocab.csv"
RACEFN = "models/fl_voter_reg/lstm/fl_ln_race.csv"

MODEL = resource_filename(__name__, MODELFN)
VOCAB = resource_filename(__name__, VOCABFN)
RACE = resource_filename(__name__, RACEFN)

NGRAMS = 2
FEATURE_LEN = 20


class FloridaRegLnModel:
    vocab = None
    race = None
    model = None

    @classmethod
    def pred_fl_reg_ln(cls, df, namecol, num_iter=100, conf_int=1):
        """Predict the race/ethnicity by the last name using Florida voter
        model.

        Using the Florida voter last name model to predict the race/ethnicity
        oF the input DataFrame.

        Args:
            df (:obj:`DataFrame`): Pandas DataFrame containing the last name
                column.
            namecol (str or int): Column's name or location of the name in
                DataFrame.

        Returns:
            DataFrame: Pandas DataFrame with additional columns:
                - `race` the predict result
                - Additional columns for probability of each classes.

        """

        if namecol not in df.columns:
            print("No column `{0!s}` in the DataFrame".format(namecol))
            return df

        df.dropna(subset=[namecol])
        if df.shape[0] == 0:
            return df
        
        rdf = transform_and_pred(
                df=df,
                newnamecol=namecol,
                cls=cls,
                VOCAB=VOCAB,
                RACE=RACE,
                MODEL=MODEL,
                NGRAMS=NGRAMS,
                maxlen=FEATURE_LEN,
                num_iter=num_iter,
                conf_int=conf_int
            )

        return rdf

pred_fl_reg_ln = FloridaRegLnModel.pred_fl_reg_ln


def main(argv=sys.argv[1:]):
    title = "Predict Race/Ethnicity by name using Florida registration model"
    parser = argparse.ArgumentParser(description=title)
    parser.add_argument("input", default=None, help="Input file")
    parser.add_argument(
        "-o",
        "--output",
        default="fl-pred-ln-output.csv",
        help="Output file with prediction data",
    )
    parser.add_argument(
        "-l",
        "--last",
        required=True,
        help="Name or index location of column contains " "the last name",
    )
    parser.add_argument('-i', '--iter', default=100, type=int,
                        help='Number of iterations to measure uncertainty')
    parser.add_argument('-c', '--conf', default=1.0, type=float,
                        help='Confidence interval of Predictions')

    args = parser.parse_args(argv)

    print(args)

    if not args.last.isdigit():
        df = pd.read_csv(args.input)
    else:
        df = pd.read_csv(args.input, header=None)
        args.last = int(args.last)

    if not column_exists(df, args.last):
        return -1

    rdf = pred_fl_reg_ln(df, args.last, args.iter, args.conf)

    print("Saving output to file: `{0:s}`".format(args.output))
    rdf.columns = fixup_columns(rdf.columns)
    rdf.to_csv(args.output, index=False)

    return 0


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import argparse
import pandas as pd

from pkg_resources import resource_filename

from .utils import column_exists, fixup_columns

CENSUS2000 = resource_filename(__name__, "data/census/census_2000.csv")
CENSUS2010 = resource_filename(__name__, "data/census/census_2010.csv")

CENSUS_COLS = ['pctwhite', 'pctblack', 'pctapi', 'pctaian', 'pct2prace',
               'pcthispanic']


class CensusLnData():
    census_df = None

    @classmethod
    def census_ln(cls, df, namecol, year=2000):
        """Appends additional columns from Census data to the input DataFrame
        based on the last name.

        Removes extra space. Checks if the name is the Census data.  If it is,
        outputs data from that row.

        Args:
            df (:obj:`DataFrame`): Pandas DataFrame containing the last name
                column.
            namecol (str or int): Column's name or location of the name in
                DataFrame.
            year (int): The year of Census data to be used. (2000 or 2010)
                (default is 2000)

        Returns:
            DataFrame: Pandas DataFrame with additional columns 'pctwhite',
                'pctblack', 'pctapi', 'pctaian', 'pct2prace', 'pcthispanic'

        """

        if namecol not in df.columns:
            print("No column `{0!s}` in the DataFrame".format(namecol))
            return df

        df['__last_name'] = df[namecol].str.strip().str.upper()

        if cls.census_df is None or cls.census_year != year:
            if year == 2000:
                cls.census_df = pd.read_csv(CENSUS2000, usecols=['name']
                                            + CENSUS_COLS)
            elif year == 2010:
                cls.census_df = pd.read_csv(CENSUS2010, usecols=['name']
                                            + CENSUS_COLS)

            cls.census_df.drop(cls.census_df[cls.census_df.name.isnull()]
                               .index, inplace=True)

            cls.census_df.columns = ['__last_name'] + CENSUS_COLS
            cls.census_year = year

        rdf = pd.merge(df, cls.census_df, how='left', on='__last_name')

        del df['__last_name']
        del rdf['__last_name']

        return rdf


census_ln = CensusLnData.census_ln


def main(argv=sys.argv[1:]):
    title = 'Appends Census columns by last name'
    parser = argparse.ArgumentParser(description=title)
    parser.add_argument('input', default=None,
                        help='Input file')
    parser.add_argument('-y', '--year', type=int, default=2000,
                        choices=[2000, 2010],
                        help='Year of Census data (default=2000)')
    parser.add_argument('-o', '--output', default='census-output.csv',
                        help='Output file with Census data columns')
    parser.add_argument('-l', '--last', required=True,
                        help='Name or index location of column contains '
                             'the last name')

    args = parser.parse_args(argv)

    print(args)

    if not args.last.isdigit():
        df = pd.read_csv(args.input)
    else:
        df = pd.read_csv(args.input, header=None)
        args.last = int(args.last)

    if not column_exists(df, args.last):
        return -1

    rdf = census_ln(df, args.last, args.year)

    print("Saving output to file: `{0:s}`".format(args.output))
    rdf.columns = fixup_columns(rdf.columns)
    rdf.to_csv(args.output, index=False)

    return 0


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Tests for pred_census_ln.py

"""

import os
import shutil
import unittest
import pandas as pd
from ethnicolr.pred_census_ln import pred_census_ln
from . import capture

race = ["api", "black", "hispanic", "white"]
race_mean = ["api_mean", "black_mean", "hispanic_mean", "white_mean"]


class TestCensusLn(unittest.TestCase):
    def setUp(self):
        names = [
            {"last": "smith", "true_race": "white"},
            {"last": "zhang", "true_race": "api"},
        ]
        self.df = pd.DataFrame(names)

    def tearDown(self):
        pass

    def test_pred_census_ln_2000(self):
        odf = pred_census_ln(self.df, "last", 2000)
        self.assertTrue(
            all(
                odf[[col for col in odf.columns
                     if col in race]].sum(axis=1).round(1)
                == 1.0
            )
        )
        self.assertTrue(all(odf.true_race == odf.race))

    def test_pred_census_ln_2010(self):
        odf = pred_census_ln(self.df, "last", 2010)
        self.assertTrue(
            all(
                odf[[col for col in odf.columns
                     if col in race]].sum(axis=1).round(1)
                == 1.0
            )
        )
        self.assertTrue(all(odf.true_race == odf.race))

    def test_pred_census_ln_2000_mean(self):
        odf = pred_census_ln(self.df, "last", 2000, conf_int=0.9)
        self.assertTrue(
            all(
                odf[[col for col in odf.columns
                     if col in race_mean]].sum(axis=1).round(1)
                == 1.0
            )
        )
        self.assertTrue(all(odf.true_race == odf.race))

    def test_pred_census_ln_2010_mean(self):
        odf = pred_census_ln(self.df, "last", 2010, conf_int=0.9)
        self.assertTrue(
            all(
                odf[[col for col in odf.columns
                     if col in race_mean]].sum(axis=1).round(1)
                == 1.0
            )
        )
        self.assertTrue(all(odf.true_race == odf.race))

if __name__ == "__main__":
    unittest.main()
